#include "CubemapTest.h"

CubemapTest::CubemapTest(void):
	Device(NULL), Driver(NULL), SceneManager(NULL)
{
	
}

CubemapTest::~CubemapTest(void)
{
	if (Device)
	{
		Device->drop();
	}
}

bool CubemapTest::init()
{
	Device = createDevice( video::EDT_DIRECT3D9, dimension2d<u32>(800, 600), 32,
			false, true, false, 0); //use stencil buffer

	if (!Device)
		return false;

	Device->setWindowCaption(L"Cube map demo");

	Driver = Device->getVideoDriver();
	SceneManager = Device->getSceneManager();

	ICameraSceneNode* camera = SceneManager->addCameraSceneNodeFPS();
	camera->setPosition(vector3df(0.0f, 100.0f, 0.0f));

	//sky
	ISceneNode* skyBox = SceneManager->addSkyBoxSceneNode(
		Driver->getTexture("data/stormy_UP.jpg"),
		Driver->getTexture("data/stormy_DN.jpg"),
		Driver->getTexture("data/stormy_FR.jpg"),
		Driver->getTexture("data/stormy_BK.jpg"),
		Driver->getTexture("data/stormy_RT.jpg"),
		Driver->getTexture("data/stormy_LF.jpg")
	);

	video::IImage* posXImage = Driver->createImageFromFile("data/stormy_RT.jpg");
	video::IImage* negXImage = Driver->createImageFromFile("data/stormy_LF.jpg");
	video::IImage* posYImage = Driver->createImageFromFile("data/stormy_UP.jpg");
	video::IImage* negYImage = Driver->createImageFromFile("data/stormy_DN.jpg");
	video::IImage* posZImage = Driver->createImageFromFile("data/stormy_FR.jpg");
	video::IImage* negZImage = Driver->createImageFromFile("data/stormy_BK.jpg");

	ITexture* cubeMap = Driver->addTextureCube("sky", posXImage, negXImage, posYImage, negYImage, posZImage, negZImage);

	loadShaders();

	scene::IMeshSceneNode* sphere = SceneManager->addSphereSceneNode(60);

	sphere->setMaterialTexture(0, cubeMap);
	sphere->setMaterialType((video::E_MATERIAL_TYPE)Shader);

	return true;
}

void CubemapTest::run()
{
	while(Device->run())
	{
		Driver->beginScene(true, true, SColor(255, 255, 255, 255));

		SceneManager->drawAll();

		Driver->endScene();
	}
}

void CubemapTest::OnSetConstants(video::IMaterialRendererServices* services, s32 userData)
{
	core::matrix4 projection = Driver->getTransform(video::ETS_PROJECTION);
	core::matrix4 view = Driver->getTransform(video::ETS_VIEW);
	core::matrix4 world = Driver->getTransform(video::ETS_WORLD);

	//calculate world view projection matrix
	core::matrix4 worldViewProj = projection;			
	worldViewProj *= view;
	worldViewProj *= world;
	services->setVertexShaderConstant("worldViewProj", worldViewProj.pointer(), 16);

	//get current camera
	scene::ICameraSceneNode* currentCamera = SceneManager->getActiveCamera();
	core::vector3df cameraPosition = currentCamera->getPosition();

	services->setVertexShaderConstant("cameraPosition", &cameraPosition.X, 3);

	//texture constants for GLSL
	if (Driver->getDriverType() == video::EDT_OPENGL)
	{
		s32 var0 = 0;
		services->setPixelShaderConstant("cubeMap", (float*)(&var0), 1); //the cubemap 
	}
}

void CubemapTest::loadShaders()
{
	//load shaders
	video::IGPUProgrammingServices* GPUProgrammingServices = Driver->getGPUProgrammingServices();

	core::stringc pixelShaderFile;
	core::stringc vertexShaderFile;

	if (Driver->getDriverType() == video::EDT_DIRECT3D9)
	{
		pixelShaderFile = "shaders/ps.hlsl";
		vertexShaderFile = "shaders/vs.hlsl";
	}
	else if (Driver->getDriverType() == video::EDT_OPENGL)
	{
		pixelShaderFile = "shaders/ps.glsl";
		vertexShaderFile = "shaders/vs.glsl";
	}

	Shader = GPUProgrammingServices->addHighLevelShaderMaterialFromFiles(
		vertexShaderFile.c_str(), "main", video::EVST_VS_2_0,
		pixelShaderFile.c_str(), "main", video::EPST_PS_2_0,
		this);
}